#include <zephyr/kernel.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/drivers/pwm.h>
#include <zephyr/drivers/uart.h>
#include <zephyr/logging/log.h>
#include <stdio.h>

#define LED0_NODE DT_ALIAS(led0)

// Adjust these values based on your servo's specifications
#define SERVO_MIN_PULSE_WIDTH_NS 1000000  // Minimum pulse width for the servo in nanoseconds (e.g., 1ms)
#define SERVO_MAX_PULSE_WIDTH_NS 2000000  // Maximum pulse width for the servo in nanoseconds (e.g., 2ms)
#define SERVO_PERIOD_NS          20000000 // Period of the servo signal in nanoseconds (e.g., 20ms)

static const struct gpio_dt_spec led = GPIO_DT_SPEC_GET(LED0_NODE, gpios);

LOG_MODULE_REGISTER(main);

int main(void) {
    const struct device *pwm_dev;
    const struct device *uart_dev;
    struct uart_config uart_cfg;
    int pwm_set_ret;

    uart_dev = DEVICE_DT_GET(DT_NODELABEL(uart1));
    if (!device_is_ready(uart_dev)) {
        LOG_ERR("UART device not ready");
        return -1;
    }

    // Configure the UART device
    uart_cfg.baudrate = 115200;
    uart_cfg.parity = UART_CFG_PARITY_NONE;
    uart_cfg.stop_bits = UART_CFG_STOP_BITS_1;
    uart_cfg.data_bits = UART_CFG_DATA_BITS_8;
    uart_cfg.flow_ctrl = UART_CFG_FLOW_CTRL_NONE;

    if (uart_configure(uart_dev, &uart_cfg) < 0) {
        LOG_ERR("Failed to configure UART");
        return -1;
    }
    printk("UART device configured");


    printk("Starting MG996R Tower Pro Servo Control\n");

    if (!gpio_is_ready_dt(&led)) {
        printk("LED device not ready");
		return 0;
	}

	if (gpio_pin_configure_dt(&led, GPIO_OUTPUT_ACTIVE) < 0) {
        printk("Failed to configure LED");
		return 0;
	}

    pwm_dev = DEVICE_DT_GET(DT_NODELABEL(pca9685));
    if (!device_is_ready(pwm_dev)) {
        printk("PWM device not ready");
        // return 0;
    }

    while (1) {
        k_sleep(K_SECONDS(2));
        gpio_pin_toggle_dt(&led);
        printk("LED toggled");

        // Rotate the servo to one end (e.g., 0 degrees)
        pwm_set_ret = pwm_set(pwm_dev, 0, SERVO_PERIOD_NS, SERVO_MIN_PULSE_WIDTH_NS, 0);
        if (pwm_set_ret) {
            printk("Error setting PWM: %d\n", pwm_set_ret);
            // return 0;
        }
        k_sleep(K_SECONDS(2));

        // Rotate the servo to the other end (e.g., 180 degrees)
        pwm_set_ret = pwm_set(pwm_dev, 0, SERVO_PERIOD_NS, SERVO_MAX_PULSE_WIDTH_NS, 0);
        if (pwm_set_ret) {
            printk("Error setting PWM: %d\n", pwm_set_ret);
            // return 0;
        }
        k_sleep(K_SECONDS(2));
    }
}